@@ -7,7 +7,7 @@ use vars qw($VERSION $BASE_EXC_CLASS %CLASSES);
 
 BEGIN { $BASE_EXC_CLASS ||= 'Exception::Class::Base'; }
 
-$VERSION = '1.12';
+$VERSION = '1.19';
 
 sub import
 {
@@ -15,13 +15,24 @@ sub import
 
     local $Exception::Class::Caller = caller();
 
+    my %c;
+
     my %needs_parent;
- MAKE_CLASSES:
     while (my $subclass = shift)
     {
 	my $def = ref $_[0] ? shift : {};
 	$def->{isa} = $def->{isa} ? ( ref $def->{isa} ? $def->{isa} : [$def->{isa}] ) : [];
 
+        $c{$subclass} = $def;
+    }
+
+    # We need to sort by length because if we check for keys in the
+    # Foo::Bar:: stash, this creates a "Bar::" key in the Foo:: stash!
+ MAKE_CLASSES:
+    foreach my $subclass ( sort { length $a <=> length $b } keys %c )
+    {
+        my $def = $c{$subclass};
+
 	# We already made this one.
 	next if $CLASSES{$subclass};
 
@@ -163,7 +174,7 @@ EOPERL
 package Exception::Class::Base;
 
 use Class::Data::Inheritable;
-use Devel::StackTrace;
+use Devel::StackTrace 1.07;
 
 use base qw(Class::Data::Inheritable);
 
@@ -173,8 +184,11 @@ BEGIN
     *do_trace = \&Trace;
     __PACKAGE__->mk_classdata('NoRefs');
     *NoObjectRefs = \&NoRefs;
+    __PACKAGE__->NoRefs(1);
+
+    __PACKAGE__->mk_classdata('RespectOverload');
+    __PACKAGE__->RespectOverload(0);
 
-    __PACKAGE__->NoRefs(0);
 
     sub Fields { () }
 }
@@ -253,12 +267,19 @@ sub _initialize
     $self->{gid}  = $(;
     $self->{egid} = $);
 
-    my $x = 0;
-    # move back the stack til we're out of this package
-    $x++ while defined caller($x) && caller($x)->isa( __PACKAGE__ );
-    $x-- until caller($x);
+    $self->{trace} =
+        Devel::StackTrace->new( ignore_class     => __PACKAGE__,
+                                ignore_package   => 'Exception::Class',
+                                no_refs          => $self->NoRefs,
+                                respect_overload => $self->RespectOverload,
+                              );
 
-    @{ $self }{ qw( package file line ) } = (caller($x))[0..2];
+    if ( my $frame = $self->trace->frame(0) )
+    {
+        $self->{package} = $frame->package;
+        $self->{line} = $frame->line;
+        $self->{file} = $frame->filename;
+    }
 
     my %fields = map { $_ => 1 } $self->Fields;
     while ( my ($key, $value) = each %p )
@@ -276,11 +297,6 @@ sub _initialize
                  "unknown field $key passed to constructor for class " . ref $self );
        }
     }
-
-    $self->{trace} =
-        Devel::StackTrace->new( ignore_class => __PACKAGE__,
-                                no_refs => $self->NoRefs,
-                              );
 }
 
 sub description
@@ -363,23 +379,23 @@ Exception::Class - A module that allows you to declare real exception classes in
       );
 
   # try
-  eval { MyException->throw( error => 'I feel funny.'; };
+  eval { MyException->throw( error => 'I feel funny.' ) };
 
   # catch
-  if ( $@ && ref $@ && $@->isa('MyException') )
+  if ( UNIVERSAL::isa( $@, 'MyException' ) )
   {
      warn $@->error, "\n, $@->trace->as_string, "\n";
      warn join ' ',  $@->euid, $@->egid, $@->uid, $@->gid, $@->pid, $@->time;
 
      exit;
   }
-  elsif ( $@->isa('ExceptionWithFields') )
+  elsif ( UNIVERSAL::isa( $@, 'ExceptionWithFields' ) )
   {
      $@->quixotic ? do_something_wacky() : do_something_sane();
   }
   else
   {
-     $@->rethrow;
+     ref $@ ? $@->rethrow : die $@;
   }
 
   # use an alias - without parens subroutine name is checked at
@@ -431,8 +447,8 @@ This can be any other C<Exception::Class::Base> subclass in your
 declaration I<or> a subclass loaded from a module.
 
 To change the default exception class you will need to change the
-value of C<$Exception::Class::BASE_EXC_CLASS> I<before> calling C<import>.
-To do this simply do something like this:
+value of C<$Exception::Class::BASE_EXC_CLASS> I<before> calling
+C<import()>.  To do this simply do something like this:
 
 BEGIN { $Exception::Class::BASE_EXC_CLASS = 'SomeExceptionClass'; }
 
@@ -448,10 +464,10 @@ error.
 =item * fields
 
 This allows you to define additional attributes for your exception
-class.  Any field you define can be passed to the C<throw> or C<new>
-methods as additional parameters for the constructor.  In addition,
-your exception object will have an accessor method for the fields you
-define.
+class.  Any field you define can be passed to the C<throw()> or
+C<new()> methods as additional parameters for the constructor.  In
+addition, your exception object will have an accessor method for the
+fields you define.
 
 This parameter can be either a scalar (for a single field) or an array
 reference if you need to define multiple fields.
@@ -516,10 +532,10 @@ This is a class method, not an object method.
 
 =item * NoRefs($boolean)
 
-When a C<Devel::StackTrace> is created, it walks through the stack and
-stores the arguments which were passed to each subroutine on the
-stack.  If any of these arguments are references, then that means that
-the C<Devel::StackTrace> ends up increasing the refcount of these
+When a C<Devel::StackTrace> object is created, it walks through the
+stack and stores the arguments which were passed to each subroutine on
+the stack.  If any of these arguments are references, then that means
+that the C<Devel::StackTrace> ends up increasing the refcount of these
 references, delaying their destruction.
 
 Since C<Exception::Class::Base> uses C<Devel::StackTrace> internally,
@@ -527,10 +543,28 @@ this method provides a way to tell C<Devel::StackTrace> not to store
 these references.  Instead, C<Devel::StackTrace> replaces references
 with their stringified representation.
 
-This method defaults to false.  As with C<Trace>, it is inherited by
+This method defaults to true.  As with C<Trace()>, it is inherited by
 subclasses but setting it in a subclass makes it independent
 thereafter.
 
+=item * RespectOverload($boolean)
+
+When a C<Devel::StackTrace> object stringifies, by default it ignores
+stringification overloading on any objects being dealt with.
+
+Since C<Exception::Class::Base> uses C<Devel::StackTrace> internally,
+this method provides a way to tell C<Devel::StackTrace> to respect
+overloading.
+
+This method defaults to false.  As with C<Trace()>, it is inherited by
+subclasses but setting it in a subclass makes it independent
+thereafter.
+
+=item * Fields
+
+This method returns the extra fields defined for the given class, as
+an array.
+
 =item * throw( $message )
 
 =item * throw( message => $message )
@@ -543,8 +577,8 @@ then die's with this object as its argument.
 
 This method also takes a C<show_trace> parameter which indicates
 whether or not the particular exception object being created should
-show a stacktrace when its C<as_string> method is called.  This
-overrides the value of C<Trace> for this class if it is given.
+show a stacktrace when its C<as_string()> method is called.  This
+overrides the value of C<Trace()> for this class if it is given.
 
 If only a single value is given to the constructor it is assumed to be
 the message parameter.
@@ -554,9 +588,12 @@ exception subclass will also be accepted.
 
 =item * new
 
-This method takes the same parameters as C<throw>, but instead of
+This method takes the same parameters as C<throw()>, but instead of
 dying simply returns a new exception object.
 
+This method is always called when constructing a new exception object
+via the C<throw()> method.
+
 =item * description
 
 Returns the description for the given C<Exception::Class::Base>
@@ -574,7 +611,7 @@ object method.
 
 Simply dies with the object as its sole argument.  It's just syntactic
 sugar.  This does not change any of the object's attribute values.
-However, it will cause C<caller> to report the die as coming from
+However, it will cause C<caller()> to report the die as coming from
 within the C<Exception::Class::Base> class rather than where rethrow
 was called.
 
@@ -639,12 +676,12 @@ stringified.
 Returns a string form of the error message (something like what you'd
 expect from die).  If the class or object is set to show traces then
 then the full trace is also included.  The result looks like
-C<Carp::confess>.
+C<Carp::confess()>.
 
 =item * full_message
 
-Called by the C<as_string> method to get the message.  By default,
-this is the same as calling the C<message> method, but may be
+Called by the C<as_string()> method to get the message.  By default,
+this is the same as calling the C<message()> method, but may be
 overridden by a subclass.  See below for details.
 
 =back
@@ -660,21 +697,21 @@ actual object.  It also means an application or module could do this:
  $SIG{__DIE__} = sub { Exception::Class::Base->throw( error => join '', @_ ); };
 
 and this would probably not break anything (unless someone was
-expecting a different type of exception object from C<die>).
+expecting a different type of exception object from C<die()>).
 
 =head1 OVERRIDING THE as_string METHOD
 
-By default, the C<as_string> method simply returns the value
+By default, the C<as_string()> method simply returns the value
 C<message> or C<error> param plus a stack trace, if the class's
-C<Trace> method returns a true value or C<show_trace> was set when
+C<Trace()> method returns a true value or C<show_trace> was set when
 creating the exception.
 
 However, once you add new fields to a subclass, you may want to
 include those fields in the stringified error.
 
-Inside the C<as_string> method, the message (non-stack trace) portion
-of the error is generated by calling the C<full_message> method.  This
-can be easily overridden.  For example:
+Inside the C<as_string()> method, the message (non-stack trace)
+portion of the error is generated by calling the C<full_message()>
+method.  This can be easily overridden.  For example:
 
   sub full_message
   {
@@ -720,6 +757,14 @@ You may want to create a real module to subclass
 C<Exception::Class::Base> as well, particularly if you want your
 exceptions to have more methods.
 
+=head2 Subclassing Exception::Class::Base
+
+As part of your usage of C<Exception::Class>, you may want to create
+your own base exception class which subclasses
+C<Exception::Class::Base>.  You should feel free to subclass any of
+the methods documented above.  For example, you may want to subclass
+C<new()> to add additional information to your exception objects.
+
 =head1 OTHER EXCEPTION MODULES (try/catch syntax)
 
 If you are interested in adding try/catch/finally syntactic sugar to
